=== modified file 'database/schema/comments.sql'
--- database/schema/comments.sql	2005-11-05 23:57:07 +0000
+++ database/schema/comments.sql	2005-11-15 16:09:23 +0000
@@ -537,6 +537,8 @@
 
 COMMENT ON COLUMN DistroReleaseQueue.pocket IS 'This is the pocket the upload is targeted at.';
 
+COMMENT ON COLUMN DistroReleaseQueue.changesfile IS 'The changes file associated with this upload.';
+
 -- DistroReleaseQueueSource
 COMMENT ON TABLE DistroReleaseQueueSource IS 'An upload queue source package. This table stores information pertaining to the source files in an in-progress package upload.';
 

=== added file 'database/schema/patch-25-99-0.sql'
--- database/schema/patch-25-99-0.sql	1970-01-01 00:00:00 +0000
+++ database/schema/patch-25-99-0.sql	2005-11-15 16:09:23 +0000
@@ -0,0 +1,10 @@
+/*
+ * Add a .changes column to the upload queue table
+ */
+
+SET client_min_messages = ERROR;
+
+ALTER TABLE DistroReleaseQueue ADD COLUMN changesfile TEXT;
+ALTER TABLE DistroReleaseQueue ALTER COLUMN changesfile SET NOT NULL;
+
+INSERT INTO LaunchpadDatabaseRevision VALUES (25,99,0);

=== modified file 'database/schema/security.cfg'
--- database/schema/security.cfg	2005-11-07 11:22:21 +0000
+++ database/schema/security.cfg	2005-11-15 16:09:23 +0000
@@ -542,17 +542,22 @@
 public.distrorelease                    = SELECT
 public.distroarchrelease                = SELECT
 public.build                            = SELECT
-public.sourcepackagerelease             = SELECT
-public.binarypackagerelease             = SELECT
+public.sourcepackagerelease             = SELECT, UPDATE
+public.binarypackagerelease             = SELECT, UPDATE
 public.sourcepackagename                = SELECT
 public.binarypackagename                = SELECT
 public.securesourcepackagepublishinghistory = SELECT, INSERT
 public.securebinarypackagepublishinghistory = SELECT, INSERT
 public.component                        = SELECT
 public.section                          = SELECT
+public.componentselection               = SELECT
+public.sectionselection                 = SELECT
 
 # Librarian stuff
 public.libraryfilealias                 = SELECT
+public.libraryfilecontent               = SELECT
+public.sourcepackagereleasefile         = SELECT
+public.binarypackagefile                = SELECT
 
 # This group is now created automatically
 # Readonly access to everything

=== modified file 'lib/canonical/archivepublisher/nascentupload.py'
--- lib/canonical/archivepublisher/nascentupload.py	2005-11-05 14:13:23 +0000
+++ lib/canonical/archivepublisher/nascentupload.py	2005-11-15 16:09:23 +0000
@@ -1703,8 +1703,9 @@
         status = DistroReleaseQueueStatus.ACCEPTED
         if self.is_new():
             status = DistroReleaseQueueStatus.NEW
+        self.logger.debug("Creating a %s queue entry" % status.title)
         queue_root = self.distrorelease.createQueueEntry(
-            self.policy.pocket, status=status)
+            self.policy.pocket, self.changes['filecontents'], status=status)
         # Next, if we're sourceful, add a source to the queue
         if self.sourceful:
             queue_root.addSource(self.policy.sourcepackagerelease)

=== modified file 'lib/canonical/archivepublisher/template_messages.py'
--- lib/canonical/archivepublisher/template_messages.py	2005-10-21 10:45:31 +0000
+++ lib/canonical/archivepublisher/template_messages.py	2005-11-15 16:09:23 +0000
@@ -33,8 +33,8 @@
 the override file.  It is ok otherwise, so please be patient.  New
 packages are usually added to the overrides about once a week.
 
-You may have gotten the distrorelease wrong.  You'll may get warnings
-above if files already exist in other distrorelease.
+You may have gotten the distrorelease wrong.  If so, you may get warnings
+above if files already exist in other distroreleases.
 """
 
 accepted_template="""From: %(FROM)s

=== modified file 'lib/canonical/launchpad/database/distrorelease.py'
--- lib/canonical/launchpad/database/distrorelease.py	2005-11-09 21:35:41 +0000
+++ lib/canonical/launchpad/database/distrorelease.py	2005-11-15 16:09:23 +0000
@@ -565,13 +565,14 @@
             distrorelease=self, owner=owner)
         return dar
 
-    def createQueueEntry(self, pocket,
+    def createQueueEntry(self, pocket, changesfile,
                          status=DistroReleaseQueueStatus.ACCEPTED):
         """See IDistroRelease."""
 
         return DistroReleaseQueue(distrorelease=self.id,
                                   pocket=pocket,
-                                  status=status)
+                                  status=status,
+                                  changesfile=changesfile)
 
     def getQueueItems(self, status=DistroReleaseQueueStatus.ACCEPTED):
         """See IDistroRelease."""

=== modified file 'lib/canonical/launchpad/database/queue.py'
--- lib/canonical/launchpad/database/queue.py	2005-11-02 23:03:39 +0000
+++ lib/canonical/launchpad/database/queue.py	2005-11-15 16:09:23 +0000
@@ -6,10 +6,12 @@
 
 import os
 import tempfile
+import pytz
+from datetime import datetime
 
 from zope.interface import implements
 
-from sqlobject import ForeignKey, MultipleJoin
+from sqlobject import ForeignKey, MultipleJoin, StringCol
 
 from canonical.database.sqlbase import SQLBase
 from canonical.database.constants import UTC_NOW
@@ -26,6 +28,8 @@
     SecureSourcePackagePublishingHistory,
     SecureBinaryPackagePublishingHistory)
 
+from canonical.cachedproperty import cachedproperty
+
 # There are imports below in DistroReleaseQueueCustom for various bits
 # of the archivepublisher which cause circular import errors if they
 # are placed here.
@@ -57,6 +61,7 @@
     pocket = EnumCol(dbName='pocket', unique=False, default=None, notNull=True,
                      schema=PackagePublishingPocket)
 
+    changesfile = StringCol(dbName='changesfile', default=None, notNull=True)
 
     # Join this table to the DistroReleaseQueueBuild and the
     # DistroReleaseQueueSource objects which are related.
@@ -68,6 +73,78 @@
     customfiles = MultipleJoin('DistroReleaseQueueCustom',
                                joinColumn='distroreleasequeue')
 
+    @cachedproperty
+    def changesfilename(self):
+        """A changes filename to accurately represent this upload."""
+        filename = self.sourcepackagename.name + "_" + self.sourceversion + "_"
+        arch_done = False
+        if len(self.sources):
+            filename += "source"
+            arch_done = True
+        for build in self.builds:
+            if arch_done:
+                filename += "+"
+            filename += build.build.distroarchrelease.architecturetag
+            arch_done = True
+        filename += ".changes"
+        return filename
+
+    @cachedproperty
+    def datecreated(self):
+        """The date on which this queue item was created.
+
+        We look through the sources/builds of this queue item to find out
+        when we created it. This is heuristic for now but may be made into
+        a column at a later date.
+        """
+        # If we can find a source, return it
+        for source in self.sources:
+            return source.sourcepackagerelease.dateuploaded
+        # Ditto for builds
+        for build in self.builds:
+            return build.build.datecreated
+        # Strange, but there's no source or build, complain
+        raise NotFoundError()
+
+    @property
+    def age(self):
+        """See IDistroReleaseQueue"""
+        UTC = pytz.timezone('UTC')
+        now = datetime.now(UTC)
+        return now - self.datecreated
+
+
+    @cachedproperty
+    def sourcepackagename(self):
+        """The source package name related to this queue item.
+
+        We look through sources/builds to find it. This is heuristic for now
+        but may be made into a column at a later date.
+        """
+        # If there's a source, use it
+        for source in self.sources:
+            return source.sourcepackagerelease.sourcepackagename
+        # ditto builds
+        for build in self.builds:
+            return build.sourcepackagerelease.sourcepackagename
+        # strange, no source or build
+        raise NotFoundError()
+
+    @cachedproperty
+    def sourceversion(self):
+        """The source package version related to this queue item.
+
+        This is currently heuristic but may be more easily calculated later.
+        """
+        # If there's a source, use it
+        for source in self.sources:
+            return source.sourcepackagerelease.version
+        # ditto builds
+        for build in self.builds:
+            return build.sourcepackagerelease.version
+        # strange, no source or build
+        raise NotFoundError()
+        
     def realiseUpload(self, logger=None):
         """See IDistroReleaseQueue."""
         assert self.status == DistroReleaseQueueStatus.ACCEPTED

=== modified file 'lib/canonical/launchpad/interfaces/distrorelease.py'
--- lib/canonical/launchpad/interfaces/distrorelease.py	2005-11-09 21:35:41 +0000
+++ lib/canonical/launchpad/interfaces/distrorelease.py	2005-11-15 16:09:23 +0000
@@ -219,9 +219,12 @@
         DistroReleaseBinaryPackage objects that match the given text.
         """
 
-    def createQueueEntry(pocket, status=DistroReleaseQueueStatus.ACCEPTED):
+    def createQueueEntry(pocket, changesfile,
+                         status=DistroReleaseQueueStatus.ACCEPTED):
         """Create a queue item attached to this distrorelease and the given
         pocket. If status is not supplied, then default to an ACCEPTED item.
+        The changesfile argument should be the text of the .changes for this
+        upload. The contents of this may be used later.
         """
 
     def newArch(architecturetag, processorfamily, official, owner):

=== modified file 'lib/canonical/launchpad/interfaces/queue.py'
--- lib/canonical/launchpad/interfaces/queue.py	2005-10-27 23:49:18 +0000
+++ lib/canonical/launchpad/interfaces/queue.py	2005-11-15 16:09:23 +0000
@@ -12,7 +12,7 @@
     ]
 
 from zope.schema import Int
-from zope.interface import Interface
+from zope.interface import Interface, Attribute
 from zope.i18nmessageid import MessageIDFactory
 
 _ = MessageIDFactory('launchpad')
@@ -36,6 +36,20 @@
             title=_("The pocket"), required=True, readonly=False,
             )
 
+    changesfile = Attribute("The changes file associated with this upload")
+    changesfilename = Attribute("The filename of the changes file.")
+    
+    sources = Attribute("The queue sources associated with this queue item")
+    builds = Attribute("The queue builds associated with the queue item")
+
+    datecreated = Attribute("The date on which this queue was created.")
+
+    sourcepackagename = Attribute("The source package name for this item.")
+
+    sourceversion = Attribute("The source package version for this item")
+
+    age = Attribute("The age of this queue item. (now - datecreated)")
+
     def realiseUpload(logger=None):
         """Take this ACCEPTED upload and create the publishing records for it
         as appropriate.

=== added directory 'scripts/ftpmaster-tools'
=== added file 'scripts/ftpmaster-tools/_pythonpath.py'
--- scripts/ftpmaster-tools/_pythonpath.py	1970-01-01 00:00:00 +0000
+++ scripts/ftpmaster-tools/_pythonpath.py	2005-11-15 16:09:23 +0000
@@ -0,0 +1,9 @@
+# Copyright 2004-2005 Canonical Ltd.  All rights reserved.
+
+__metaclass__ = type
+
+import sys, os, os.path
+
+sys.path.insert(0, os.path.join(
+    os.path.dirname(__file__), os.pardir, os.pardir, 'lib'
+    ))

=== added file 'scripts/ftpmaster-tools/queue'
--- scripts/ftpmaster-tools/queue	1970-01-01 00:00:00 +0000
+++ scripts/ftpmaster-tools/queue	2005-11-15 16:09:23 +0000
@@ -0,0 +1,423 @@
+#!/usr/bin/python
+
+# Copyright 2005 Canonical Ltd
+
+"""Queue management script
+
+XXX: This is a temporary measure until queue visualisation and manipulation
+is migrated into the launchpad web-UI. Don't look too closely at it thus.
+
+Yes the script is vile, yes it is nasty, no I'm not going to make it less
+vile until we migrate the functionality into the UI whereupon this script
+can be removed :-)
+"""
+
+import _pythonpath
+
+import os
+import sys
+from fnmatch import fnmatch
+
+from optparse import OptionParser
+from zope.component import getUtility
+
+from canonical.lp import initZopeless
+from canonical.config import config
+from canonical.launchpad.scripts import (execute_zcml_for_scripts,
+                                         logger, logger_options)
+
+from canonical.launchpad.interfaces import IDistributionSet
+from canonical.lp.dbschema import (
+    DistroReleaseQueueStatus, PackagePublishingPriority)
+from canonical.launchpad.webapp.tales import DurationFormatterAPI
+
+name_queue_map = {
+    "new": DistroReleaseQueueStatus.NEW,
+    "unapproved": DistroReleaseQueueStatus.UNAPPROVED,
+    "accepted": DistroReleaseQueueStatus.ACCEPTED,
+    "done": DistroReleaseQueueStatus.DONE,
+    "rejected": DistroReleaseQueueStatus.REJECTED
+    }
+
+queue_name_map = dict((v, k) for (k, v) in name_queue_map.items())
+
+name_priority_map = {
+    'required': PackagePublishingPriority.REQUIRED,
+    'important': PackagePublishingPriority.IMPORTANT,
+    'standard': PackagePublishingPriority.STANDARD,
+    'optional': PackagePublishingPriority.OPTIONAL,
+    'extra': PackagePublishingPriority.EXTRA,
+    '': None
+    }
+
+def compare_queue_by_date(a, b):
+    return cmp(a.datecreated, b.datecreated)
+
+def filechunks(file, chunk_size=256*1024):
+    """Return an iterator which reads chunks of the given file."""
+    # We use the two-arg form of the iterator here to form an iterator
+    # which reads chunks from the given file.
+    return iter(lambda: file.read(chunk_size), '')
+
+def filter_queue(queue, must_filter=False):
+    """Filter the queue, if must_filter is False then we return the entire
+    queue should no filter be supplied.
+    """
+    if not must_filter and len(args) < 2:
+        return queue
+    if len(args) < 2:
+        print "Unable to filter queue. No filter provided. If you really meant"
+        print "to act on everything then provide the filter '*'"
+        sys.exit(1)
+    full_filter = args[1]
+    if full_filter == "*":
+        return queue
+    if "/" in full_filter:
+        (spn_filter, vers_filter) = full_filter.split("/")
+    else:
+        (spn_filter, vers_filter) = (full_filter, "*")
+
+    if spn_filter.isdigit():
+        for queue_item in queue:
+            if queue_item.id == int(spn_filter):
+                return [queue_item]
+        print "Unable to find queue item %d" % int(spn)
+        sys.exit(1)
+
+    ret_queue = []
+    for queue_item in queue:
+        if (fnmatch(queue_item.sourcepackagename.name, spn_filter) and
+            fnmatch(queue_item.sourceversion, vers_filter)):
+            ret_queue.append(queue_item)
+    if len(ret_queue) == 0:
+        print "Unable to find any queue items matching filter %s" % full_filter
+        sys.exit(1)
+
+    return ret_queue
+
+def visualise_item(queue_item):
+    """Print out a one line summary of the queue item provided."""
+    spn = queue_item.sourcepackagename.name
+    sourceful = len(queue_item.sources) > 0
+    binaryful = len(queue_item.builds) > 0
+    sourcever = queue_item.sourceversion
+    age = DurationFormatterAPI(queue_item.age).approximateduration()
+    vis_item = "%8d | " % queue_item.id
+    if sourceful:
+        vis_item += "S"
+    else:
+        vis_item += "-"
+    if binaryful:
+        vis_item += "B | "
+    else:
+        vis_item += "- | "
+    if len(spn) > 20:
+        spn = spn[:20]
+    else:
+        spn += " " * (20 - len(spn))
+    vis_item += spn + " | "
+    sourcever = str(sourcever)
+    if len(sourcever) > 20:
+        sourcever = sourcever[:20]
+    else:
+        sourcever += " " * (20 - len(sourcever))
+    vis_item += sourcever + " | " + age
+    print vis_item
+
+def action_LIST(distro, distrorelease, queue):
+    """List the queue in date order, oldest first."""
+    print "Listing %s/%s (%s)" % (options.distro, options.distrorelease,
+                                  options.queue)
+    print "-" * 9 + "|----|" + "-" * 22 + "|" + "-" * 22 + "|" + "-" * 15
+    filtered_queue = filter_queue(queue)
+    for queue_item in filtered_queue:
+        visualise_item(queue_item)
+    print "-" * 9 + "|----|" + "-" * 22 + "|" + "-" * 22 + "|" + "-" * 15
+    print " " * (9+6+1+22+1+22+2) + "%d/%d total" % (
+        len(filtered_queue), len(queue))
+
+def action_FETCH(distro, distrorelease, queue):
+    """Fetch the items (by filter) to CWD"""
+    filtered_queue = filter_queue(queue, True)
+    print "Fetching from %s/%s (%s) filtering on %s" % (
+        options.distro, options.distrorelease, options.queue, args[1])
+    print "Downloading %d/%d items from the queue." % (
+        len(filtered_queue), len(queue))
+    for queue_item in filtered_queue:
+        print "Constructing %s" % queue_item.changesfilename
+        changes_file = open(queue_item.changesfilename, "w")
+        changes_file.write(queue_item.changesfile)
+        changes_file.close()
+        L = []
+        for source in queue_item.sources:
+            L.extend(source.sourcepackagerelease.files)
+        for build in queue_item.builds:
+            for bpr in build.build.binarypackages:
+                L.extend(bpr.files)
+        for file_ref in L:
+            libfile = file_ref.libraryfile
+            print "Constructing %s" % libfile.filename
+            libfile.open()
+            out_file = open(libfile.filename, "w")
+            for chunk in filechunks(libfile):
+                out_file.write(chunk)
+            out_file.close()
+            libfile.close()
+
+def action_ACCEPT(distro, distrorelease, queue):
+    """Move the specified queue items to the accepted state."""
+    filtered_queue = filter_queue(queue, True)
+    print "Accepting %d/%d items from %s/%s (%s)" % (
+        len(filtered_queue), len(queue), options.distro, options.distrorelease,
+        options.queue)
+    # XXX: dsilvers: 20051115: Should not need this, but until ZCML makes more
+    # sense to me, it's quicker than fighting the security model.
+    from zope.security.proxy import removeSecurityProxy
+    for queue_item in filtered_queue:
+        removeSecurityProxy(queue_item).status = \
+            DistroReleaseQueueStatus.ACCEPTED
+        # XXX: Send accept and announce mails here. May not commit without
+        # fixing this.
+
+    return True # Tell the main loop to commit.
+
+def action_REJECT(distro, distrorelease, queue):
+    """Move the specified queue items to the accepted state."""
+    filtered_queue = filter_queue(queue, True)
+    if len(args) < 3:
+        print "Need a rejection message as arg 3"
+        sys.exit(1)
+    print "Rejecting %d/%d items from %s/%s (%s)" % (
+        len(filtered_queue), len(queue), options.distro, options.distrorelease,
+        options.queue)
+    # XXX: dsilvers: 20051115: Should not need this, but until ZCML makes more
+    # sense to me, it's quicker than fighting the security model.
+    from zope.security.proxy import removeSecurityProxy
+    for queue_item in filtered_queue:
+        removeSecurityProxy(queue_item).status = \
+            DistroReleaseQueueStatus.REJECTED
+        # XXX: Send reject mails here. May not commit without fixing this.
+
+    return True # Tell the main loop to commit.
+
+def action_INFO(distro, distrorelease, queue):
+    """Display information about a queue item."""
+    filtered_queue = filter_queue(queue, True)
+    for queue_item in filtered_queue:
+        visualise_item(queue_item)
+        for source in queue_item.sources:
+            spr = source.sourcepackagerelease
+            print "Source: %s/%s Component: %s Section: %s" % (
+                spr.sourcepackagename.name, spr.version, spr.component.name,
+                spr.section.name)
+        for build in queue_item.builds:
+            for bpr in build.build.binarypackages:
+                print ("Binary: %s/%s/%s Component: %s Section: %s "
+                       "Priority: %s") % (
+                    bpr.binarypackagename.name, bpr.version,
+                    build.build.distroarchrelease.architecturetag,
+                    bpr.component.name, bpr.section.name, bpr.priority.name)
+
+def invalid_overrides():
+    """Explain the override usage."""
+    print """Override usage:
+
+queue override <filter> override_stanza [override_stanza*]
+
+Where override_stanza is one of:
+    source [<component>]/[<section>]
+    binary <binaryname> [<component>]/[<section>]/[<priority>]
+
+In each case, when you want to leave an override alone leave it blank.
+
+So, to set the binary 'ed' to have section 'editors' but leave the component
+and priority alone, do:
+
+queue override <filter> binary ed /editors/
+
+Or, to set ed's source's section to editors, do:
+
+queue override <filter> source /editors
+"""
+    sys.exit(1)
+
+def parse_overrides(distrorelease):
+    """Parse the argument vector for overrides."""
+    if len(args) < 4:
+        invalid_overrides()
+    ret = {}
+    chompy_args = list(args[2:])
+    while len(chompy_args):
+        override_type = chompy_args.pop(0)
+        if 'source'.startswith(override_type):
+            pkg_name = "_source"
+            ret[pkg_name] = chompy_args.pop(0).split("/")
+            if len(ret[pkg_name]) != 2:
+                print "Incorrect number of elements for source overrides."
+                print "Expected 2, got %d" % len(ret[pkg_name])
+                invalid_overrides()
+            found=ret[pkg_name][0] == ''
+            for comp in distrorelease.components:
+                if comp.name == ret[pkg_name][0]:
+                    ret[pkg_name][0] = comp
+                    found=True
+            if not found:
+                print "Unknown component: %s" % ret[pkg_name][0]
+                sys.exit(1)
+            found=ret[pkg_name][1] == ''
+            for section in distrorelease.sections:
+                if section.name == ret[pkg_name][1]:
+                    ret[pkg_name][1] = section
+                    found = True
+            if not found:
+                print "Unknown section: %s" % ret[pkg_name][1]
+                sys.exit(1)
+        elif 'binary'.startswith(override_type):
+            pkg_name = chompy_args.pop(0)
+            ret[pkg_name] = chompy_args.pop(0).split("/")
+            if len(ret[pkg_name]) != 3:
+                print "Incorrect number of elements for binary overrides."
+                print "Expected 3, got %d" % len(ret[pkg_name])
+                invalid_overrides()
+            found=ret[pkg_name][0] == ''
+            for comp in distrorelease.components:
+                if comp.name == ret[pkg_name][0]:
+                    found=True
+                    ret[pkg_name][0] = comp
+            if not found:
+                print "Unknown component: %s" % ret[pkg_name][0]
+                sys.exit(1)
+            found=ret[pkg_name][1] == ''
+            for section in distrorelease.sections:
+                if section.name == ret[pkg_name][1]:
+                    ret[pkg_name][1] = section
+                    found = True
+            if not found:
+                print "Unknown section: %s" % ret[pkg_name][1]
+                sys.exit(1)
+            if ret[pkg_name][2] not in name_priority_map:
+                print "Unknown priority: %s" % ret[pkg_name][2]
+                sys.exit(1)
+            ret[pkg_name][2] = name_priority_map[ret[pkg_name][2]]
+        else:
+            print "Unknown override type: %s" % override_type
+            print "Override types known: source, binary"
+            invalid_overrides()
+    return ret
+
+def _update_source_overrides(spr, parsed_overrides):
+    """Update the overrides on *this* source package release"""
+    if "_source" not in parsed_overrides:
+        return
+    overrides = parsed_overrides["_source"]
+    if overrides[0] != '':
+        spr.component = overrides[0]
+    if overrides[1] != '':
+        spr.section = overrides[1]
+
+def _update_binary_overrides(bpr, parsed_overrides):
+    """Update the overrides on *this* binary package release"""
+    if str(bpr.binarypackagename.name) not in parsed_overrides:
+        return
+    overrides = parsed_overrides[str(bpr.binarypackagename.name)]
+    if overrides[0] != '':
+        bpr.component = overrides[0]
+    if overrides[1] != '':
+        bpr.section = overrides[1]
+    if overrides[2] is not None:
+        bpr.priority = overrides[2]
+    
+def action_OVERRIDE(distro, distrorelease, queue):
+    """Alter queued overrides."""
+    filtered_queue = filter_queue(queue, True)
+    parsed_overrides = parse_overrides(distrorelease)
+    # XXX: dsilvers: 20051115: Should not need this, but until ZCML makes more
+    # sense to me, it's quicker than fighting the security model.
+    from zope.security.proxy import removeSecurityProxy
+    print "Altering overrides on %d/%d items from %s/%s (%s)" % (
+        len(filtered_queue), len(queue), options.distro, options.distrorelease,
+            options.queue)
+    for queue_item in filtered_queue:
+        for source in queue_item.sources:
+            _update_source_overrides(
+                removeSecurityProxy(source.sourcepackagerelease),
+                parsed_overrides)
+        for build in queue_item.builds:
+            for bpr in build.build.binarypackages:
+                _update_binary_overrides(removeSecurityProxy(bpr),
+                                         parsed_overrides)
+
+    print "Running INFO for results"
+    action_INFO(distro, distrorelease, queue)
+    return True
+
+def main():
+    # Parse command-line arguments
+    parser = OptionParser()
+    logger_options(parser)
+
+    parser.add_option("-Q", "--queue",
+                      dest="queue", metavar="QUEUE", default="new",
+                      help="Which queue to consider")
+    parser.add_option("-D", "--distro",
+                      dest="distro", metavar="DISTRO", default="ubuntu",
+                      help="Which distro to look in")
+    parser.add_option("-R", "--distrorelease",
+                      dest="distrorelease", metavar="DISTRORELEASE",
+                      default="dapper",
+                      help="Which distrorelease to look in")
+
+    parser.add_option("-A", "--announcelist",
+                      dest="announcelist", metavar="ANNOUNCELIST",
+                      default=None,
+                      help="Overrides the announcement list for accepts.")
+    
+    global options
+    global args
+    (options, args) = parser.parse_args()
+
+    global log
+    log = logger(options, "queue")
+
+    if len(args) == 0:
+        log.error("Action not supplied.")
+        log.info("Valid actions are:")
+        for action_func_name, body in [(fname, func) for (fname, func) in
+                                       globals().items()
+                                       if fname.startswith("action_")]:
+            action = action_func_name.split("_",1)[1]
+            what_it_does = body.__doc__.split("\n")[0]
+            log.info("%s: %s" % (action.lower(), what_it_does))
+        return 1
+
+    action = args[0].upper()
+
+    if "action_"+action not in globals():
+        log.error("Unable to map action %s" % action)
+        return 1
+
+    if options.queue not in name_queue_map:
+        log.error("Unable to map queue name %s" % options.queue)
+        return 1
+
+    log.debug("Initialising connection to queue %s/%s" % (
+        options.distro, options.distrorelease))
+
+    global ztm
+    ztm = initZopeless(dbuser=config.uploadqueue.dbuser)
+
+    execute_zcml_for_scripts()
+
+    distro = getUtility(IDistributionSet).getByName(options.distro)
+    distrorelease = distro[options.distrorelease]
+    queue = distrorelease.getQueueItems(name_queue_map[options.queue])
+
+    actualised_queue = list(queue)
+    actualised_queue.sort(cmp=compare_queue_by_date)
+
+    if globals()["action_"+action](distro, distrorelease, actualised_queue):
+        ztm.commit()
+    
+if __name__ == '__main__':
+    sys.exit(main())
+

=== modified file 'scripts/process-upload.py'
--- scripts/process-upload.py	2005-10-26 11:30:28 +0000
+++ scripts/process-upload.py	2005-11-15 16:09:23 +0000
@@ -37,6 +37,11 @@
     parser.add_option("-N", "--dry-run", action="store_true",
                       dest="dryrun", metavar="DRY_RUN", default=False,
                       help="Whether to treat this as a dry-run or not.")
+
+    parser.add_option("-M", "--no-mails", action="store_true",
+                      dest="nomails", metavar="NO_MAILS", default=False,
+                      help="Whether to suppress the sending of mails or not.")
+    
     global options
     (options, args) = parser.parse_args()
 
@@ -95,7 +100,7 @@
             log.debug("This will cause the sendmail() to assert.")
             print repr(mail_text)
         mail_message['X-Katie'] = "Launchpad actually"
-        if options.dryrun:
+        if options.dryrun or options.nomails:
             log.info("Would be sending a mail:")
             log.info("   Subject: %s" % mail_message['Subject'])
             log.info("   Recipients: %s" % mail_message['To'])
@@ -130,7 +135,7 @@
             log.info("Dry run, aborting the transaction for this upload.")
             ztm.abort()
         else:
-            log.info("Committing the transaction and any mails associated"
+            log.info("Committing the transaction and any mails associated "
                      "with this upload.")
             ztm.commit()
     finally:

